@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -65,6 +65,7 @@
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/memory-private.h"
 #include "magick/module.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
@@ -102,6 +103,7 @@
 #define IPTC_MARKER  (JPEG_APP0+13)
 #define XML_MARKER  (JPEG_APP0+1)
 #define MaxBufferExtent  16384
+#define MaxJPEGScans  1024
 
 /*
   Typedef declarations.
@@ -167,6 +169,13 @@ typedef struct _QuantizationTable
     *levels;
 } QuantizationTable;
 
+/*
+  Const declarations.
+*/
+static const char
+  xmp_namespace[] = "http://ns.adobe.com/xap/1.0/ ";
+#define XmpNamespaceExtent 28
+
 /*
   Forward declarations.
 */
@@ -305,22 +314,49 @@ static void JPEGErrorHandler(j_common_ptr jpeg_info)
   ErrorManager
     *error_manager;
 
+  ExceptionInfo
+    *exception;
+
   Image
     *image;
 
   *message='\0';
   error_manager=(ErrorManager *) jpeg_info->client_data;
   image=error_manager->image;
+  exception=(&image->exception);
   (jpeg_info->err->format_message)(jpeg_info,message);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
       "[%s] JPEG Trace: \"%s\"",image->filename,message);
   if (error_manager->finished != MagickFalse)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),
-      CorruptImageWarning,(char *) message,"`%s'",image->filename);
+    (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageWarning,
+      (char *) message,"`%s'",image->filename);
   else
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),
-      CorruptImageError,(char *) message,"`%s'",image->filename);
+    (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageError,
+      (char *) message,"`%s'",image->filename);
+  longjmp(error_manager->error_recovery,1);
+}
+
+static void JPEGProgressHandler(j_common_ptr jpeg_info)
+{
+  ErrorManager
+    *error_manager;
+
+  ExceptionInfo
+    *exception;
+
+  Image
+    *image;
+
+  error_manager=(ErrorManager *) jpeg_info->client_data;
+  image=error_manager->image;
+  exception=(&image->exception);
+  if (jpeg_info->is_decompressor == 0)
+    return;
+  if (((j_decompress_ptr) jpeg_info)->input_scan_number < MaxJPEGScans)
+    return;
+  (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageError,
+    "too many scans","`%s'",image->filename);
   longjmp(error_manager->error_recovery,1);
 }
 
@@ -347,7 +383,7 @@ static MagickBooleanType JPEGWarningHandler(j_common_ptr jpeg_info,int level)
       */
       (jpeg_info->err->format_message)(jpeg_info,message);
       if (jpeg_info->err->num_warnings++ < JPEGExcessiveWarnings)
-        ThrowBinaryException(CorruptImageWarning,(char *) message,
+        ThrowBinaryImageException(CorruptImageWarning,(char *) message,
           image->filename);
     }
   else
@@ -613,6 +649,7 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
     *p++=(unsigned char) c;
   }
   error_manager->profile=NULL;
+  /* The IPTC profile is actually an 8bim */
   iptc_profile=(StringInfo *) GetImageProfile(image,"8bim");
   if (iptc_profile != (StringInfo *) NULL)
     {
@@ -705,7 +742,8 @@ static boolean ReadProfile(j_decompress_ptr jpeg_info)
       p=GetStringInfoDatum(profile);
       if ((length > 4) && (LocaleNCompare((char *) p,"exif",4) == 0))
         (void) CopyMagickString(name,"exif",MaxTextExtent);
-      if ((length > 5) && (LocaleNCompare((char *) p,"http:",5) == 0))
+      else if ((length > XmpNamespaceExtent) &&
+          (LocaleNCompare((char *) p,xmp_namespace,XmpNamespaceExtent-1) == 0))
         {
           ssize_t
             j;
@@ -713,8 +751,8 @@ static boolean ReadProfile(j_decompress_ptr jpeg_info)
           /*
             Extract namespace from XMP profile.
           */
-          p=GetStringInfoDatum(profile);
-          for (j=0; j < (ssize_t) GetStringInfoLength(profile); j++)
+          p=GetStringInfoDatum(profile)+XmpNamespaceExtent;
+          for (j=XmpNamespaceExtent; j < (ssize_t) GetStringInfoLength(profile); j++)
           {
             if (*p == '\0')
               break;
@@ -726,7 +764,8 @@ static boolean ReadProfile(j_decompress_ptr jpeg_info)
         }
     }
   previous_profile=GetImageProfile(image,name);
-  if (previous_profile != (const StringInfo *) NULL)
+  if ((previous_profile != (const StringInfo *) NULL) &&
+      (CompareStringInfo(previous_profile,profile) != 0))
     {
       size_t
         length;
@@ -1003,6 +1042,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     value[MaxTextExtent];
 
   const char
+    *dct_method,
     *option;
 
   ErrorManager
@@ -1039,6 +1079,9 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   struct jpeg_error_mgr
     jpeg_error;
 
+  struct jpeg_progress_mgr
+    jpeg_progress;
+
   register JSAMPLE
     *p;
 
@@ -1078,6 +1121,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   (void) memset(&error_manager,0,sizeof(error_manager));
   (void) memset(&jpeg_info,0,sizeof(jpeg_info));
   (void) memset(&jpeg_error,0,sizeof(jpeg_error));
+  (void) memset(&jpeg_progress,0,sizeof(jpeg_progress));
   jpeg_info.err=jpeg_std_error(&jpeg_error);
   jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) JPEGWarningHandler;
   jpeg_info.err->error_exit=(void (*)(j_common_ptr)) JPEGErrorHandler;
@@ -1097,6 +1141,10 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
     }
   jpeg_info.client_data=(void *) &error_manager;
   jpeg_create_decompress(&jpeg_info);
+  if (GetMaxMemoryRequest() != ~0UL)
+    jpeg_info.mem->max_memory_to_use=(long) GetMaxMemoryRequest();
+  jpeg_progress.progress_monitor=(void (*)(j_common_ptr)) JPEGProgressHandler;
+  jpeg_info.progress=(&jpeg_progress);
   JPEGSourceManager(&jpeg_info,image);
   jpeg_set_marker_processor(&jpeg_info,JPEG_COM,ReadComment);
   option=GetImageOption(image_info,"profile:skip");
@@ -1198,33 +1246,32 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   if (option != (const char *) NULL)
     jpeg_info.do_block_smoothing=IsStringTrue(option) != MagickFalse ? TRUE :
       FALSE;
-  jpeg_info.dct_method=JDCT_FLOAT;
-  option=GetImageOption(image_info,"jpeg:dct-method");
-  if (option != (const char *) NULL)
-    switch (*option)
+  dct_method=GetImageOption(image_info,"jpeg:dct-method");
+  if (dct_method != (const char *) NULL)
+    switch (*dct_method)
     {
       case 'D':
       case 'd':
       {
-        if (LocaleCompare(option,"default") == 0)
+        if (LocaleCompare(dct_method,"default") == 0)
           jpeg_info.dct_method=JDCT_DEFAULT;
         break;
       }
       case 'F':
       case 'f':
       {
-        if (LocaleCompare(option,"fastest") == 0)
+        if (LocaleCompare(dct_method,"fastest") == 0)
           jpeg_info.dct_method=JDCT_FASTEST;
-        if (LocaleCompare(option,"float") == 0)
+        if (LocaleCompare(dct_method,"float") == 0)
           jpeg_info.dct_method=JDCT_FLOAT;
         break;
       }
       case 'I':
       case 'i':
       {
-        if (LocaleCompare(option,"ifast") == 0)
+        if (LocaleCompare(dct_method,"ifast") == 0)
           jpeg_info.dct_method=JDCT_IFAST;
-        if (LocaleCompare(option,"islow") == 0)
+        if (LocaleCompare(dct_method,"islow") == 0)
           jpeg_info.dct_method=JDCT_ISLOW;
         break;
       }
@@ -1333,8 +1380,8 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   jpeg_pixels=(JSAMPLE *) GetVirtualMemoryBlob(memory_info);
-  (void) memset(jpeg_pixels,0,image->columns* 
-    jpeg_info.output_components*sizeof(*jpeg_pixels));
+  (void) memset(jpeg_pixels,0,image->columns*jpeg_info.output_components*
+    sizeof(*jpeg_pixels));
   /*
     Convert JPEG pixels to pixel packets.
   */
@@ -1532,18 +1579,23 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
 */
 ModuleExport size_t RegisterJPEGImage(void)
 {
+#define JPEGDescription "Joint Photographic Experts Group JFIF format"
+#define JPEGStringify(macro_or_string)  JPEGStringifyArg(macro_or_string)
+#define JPEGStringifyArg(contents)  #contents
+
   char
     version[MaxTextExtent];
 
   MagickInfo
     *entry;
 
-  static const char
-    description[] = "Joint Photographic Experts Group JFIF format";
-
   *version='\0';
-#if defined(JPEG_LIB_VERSION)
-  (void) FormatLocaleString(version,MaxTextExtent,"%d",JPEG_LIB_VERSION);
+#if defined(LIBJPEG_TURBO_VERSION)
+  (void) CopyMagickString(version,"libjpeg-turbo " JPEGStringify(
+    LIBJPEG_TURBO_VERSION),MagickPathExtent);
+#elif defined(JPEG_LIB_VERSION)
+  (void) FormatLocaleString(version,MagickPathExtent,"libjpeg %d",
+    JPEG_LIB_VERSION);
 #endif
   entry=SetMagickInfo("JPE");
 #if (JPEG_LIB_VERSION < 80) && !defined(LIBJPEG_TURBO_VERSION)
@@ -1556,7 +1608,7 @@ ModuleExport size_t RegisterJPEGImage(void)
   entry->magick=(IsImageFormatHandler *) IsJPEG;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=ConstantString(description);
+  entry->description=ConstantString(JPEGDescription);
   if (*version != '\0')
     entry->version=ConstantString(version);
   entry->mime_type=ConstantString("image/jpeg");
@@ -1573,7 +1625,7 @@ ModuleExport size_t RegisterJPEGImage(void)
   entry->magick=(IsImageFormatHandler *) IsJPEG;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=ConstantString(description);
+  entry->description=ConstantString(JPEGDescription);
   if (*version != '\0')
     entry->version=ConstantString(version);
   entry->mime_type=ConstantString("image/jpeg");
@@ -1589,7 +1641,7 @@ ModuleExport size_t RegisterJPEGImage(void)
 #endif
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=ConstantString(description);
+  entry->description=ConstantString(JPEGDescription);
   if (*version != '\0')
     entry->version=ConstantString(version);
   entry->mime_type=ConstantString("image/jpeg");
@@ -1605,7 +1657,7 @@ ModuleExport size_t RegisterJPEGImage(void)
 #endif
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=ConstantString(description);
+  entry->description=ConstantString(JPEGDescription);
   if (*version != '\0')
     entry->version=ConstantString(version);
   entry->mime_type=ConstantString("image/jpeg");
@@ -1621,7 +1673,7 @@ ModuleExport size_t RegisterJPEGImage(void)
 #endif
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->description=ConstantString(description);
+  entry->description=ConstantString(JPEGDescription);
   if (*version != '\0')
     entry->version=ConstantString(version);
   entry->mime_type=ConstantString("image/jpeg");
@@ -1978,6 +2030,17 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
 
     profile=GetImageProfile(image,name);
     p=GetStringInfoDatum(custom_profile);
+    length=GetStringInfoLength(profile);
+    if (LocaleNCompare(name,"APP",3) == 0)
+      {
+        int
+          id;
+
+        id=JPEG_APP0+StringToInteger(name+3);
+        for (i=0; i < (ssize_t) length; i+=65533L)
+           jpeg_write_marker(jpeg_info,id,GetStringInfoDatum(profile)+i,
+             (unsigned int) MagickMin(length-i,65533));
+      }
     if (LocaleCompare(name,"EXIF") == 0)
       {
         length=GetStringInfoLength(profile);
@@ -2050,12 +2113,12 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
         /*
           Add namespace to XMP profile.
         */
-        xmp_profile=StringToStringInfo("http://ns.adobe.com/xap/1.0/ ");
+        xmp_profile=StringToStringInfo(xmp_namespace);
         if (xmp_profile != (StringInfo *) NULL)
           {
             if (profile != (StringInfo *) NULL)
               ConcatenateStringInfo(xmp_profile,profile);
-            GetStringInfoDatum(xmp_profile)[28]='\0';
+            GetStringInfoDatum(xmp_profile)[XmpNamespaceExtent]='\0';
             for (i=0; i < (ssize_t) GetStringInfoLength(xmp_profile); i+=65533L)
             {
               length=MagickMin(GetStringInfoLength(xmp_profile)-i,65533L);
@@ -2135,6 +2198,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   Image *image)
 {
   const char
+    *dct_method,
     *option,
     *sampling_factor,
     *value;
@@ -2293,33 +2357,32 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       if (image->units == PixelsPerCentimeterResolution)
         jpeg_info.density_unit=(UINT8) 2;
     }
-  jpeg_info.dct_method=JDCT_FLOAT;
-  option=GetImageOption(image_info,"jpeg:dct-method");
-  if (option != (const char *) NULL)
-    switch (*option)
+  dct_method=GetImageOption(image_info,"jpeg:dct-method");
+  if (dct_method != (const char *) NULL)
+    switch (*dct_method)
     {
       case 'D':
       case 'd':
       {
-        if (LocaleCompare(option,"default") == 0)
+        if (LocaleCompare(dct_method,"default") == 0)
           jpeg_info.dct_method=JDCT_DEFAULT;
         break;
       }
       case 'F':
       case 'f':
       {
-        if (LocaleCompare(option,"fastest") == 0)
+        if (LocaleCompare(dct_method,"fastest") == 0)
           jpeg_info.dct_method=JDCT_FASTEST;
-        if (LocaleCompare(option,"float") == 0)
+        if (LocaleCompare(dct_method,"float") == 0)
           jpeg_info.dct_method=JDCT_FLOAT;
         break;
       }
       case 'I':
       case 'i':
       {
-        if (LocaleCompare(option,"ifast") == 0)
+        if (LocaleCompare(dct_method,"ifast") == 0)
           jpeg_info.dct_method=JDCT_IFAST;
-        if (LocaleCompare(option,"islow") == 0)
+        if (LocaleCompare(dct_method,"islow") == 0)
           jpeg_info.dct_method=JDCT_ISLOW;
         break;
       }
@@ -2453,6 +2516,8 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       jpeg_info=DestroyImageInfo(jpeg_info);
     }
   jpeg_set_quality(&jpeg_info,quality,TRUE);
+  if ((dct_method == (const char *) NULL) && (quality <= 90))
+    jpeg_info.dct_method=JDCT_IFAST;
 #if (JPEG_LIB_VERSION >= 70)
   option=GetImageOption(image_info,"quality");
   if (option != (const char *) NULL)
@@ -2484,7 +2549,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   if (value != (char *) NULL)
     colorspace=StringToInteger(value);
   sampling_factor=(const char *) NULL;
-  if (colorspace == jpeg_info.in_color_space)
+  if ((J_COLOR_SPACE) colorspace == jpeg_info.in_color_space)
     {
       value=GetImageOption(image_info,"jpeg:sampling-factor");
       if (value == (char *) NULL)
